@@ -1,9 +1,12 @@
+import canvasapi.submission
+
 import canvaslms.cli.assignments as assignments
 import canvaslms.cli.users as users
 import canvaslms.hacks.canvasapi
 
 import argparse
 import csv
+import json
 import pydoc
 import pypandoc
 import re
@@ -46,7 +49,7 @@ def speedgrader(submission):
 def submission_command(config, canvas, args):
   submission_list = process_submission_options(canvas, args)
   for submission in submission_list:
-    pydoc.pager(format_submission(submission, canvas))
+    pydoc.pager(format_submission(submission))
 def add_submission_options(parser):
   try:
     assignments.add_assignment_option(parser)
@@ -113,77 +116,76 @@ def format_submission_short_unique(submission):
   return [submission.assignment.course.course_code, submission.assignment.name,
     submission.user.login_id, submission.grade,
     submission.submitted_at, submission.graded_at]
-def format_submission(submission, canvas):
+def format_submission(submission):
   """Formats submission for printing to stdout"""
   student = submission.assignment.course.get_user(submission.user_id)
 
-  formatted_submission = f"""# Metadata
-
-{submission.assignment.course.course_code} > {submission.assignment.name}
-
-- Student: {student.name} ({student.login_id or None}, {submission.user_id})
-- Submission ID: {submission.id}
-- Submitted (graded): {submission.submitted_at} ({submission.graded_at})
-- Grade: {submission.grade} ({submission.score})
-- SpeedGrader: {speedgrader(submission)}
-"""
+  formatted_submission = ""
 
+  formatted_submission += format_section(
+    "Metadata",
+    f"{submission.assignment.course.course_code} > {submission.assignment.name}"
+    f"\n\n"
+    f" - Student: {student.name} "
+      f"({student.login_id or None}, {submission.user_id})\n"
+    f" - Submission ID: {submission.id}\n"
+    f" - Submitted (graded): {submission.submitted_at} "
+      f"({submission.graded_at})\n"
+    f" - Grade: {submission.grade} ({submission.score})\n"
+    f" - SpeedGrader: {speedgrader(submission)}")
   try:
     if submission.rubric_assessment:
-      formatted_submission += f"""
-
-# Rubric assessment
-
-{format_rubric(submission)}
-"""
+      formatted_submission += format_section(
+        "Rubric assessment",
+        format_rubric(submission))
   except AttributeError:
     pass
-
   try:
     if submission.submission_comments:
-      formatted_submission += f"""
-
-# Comments
-"""
-
-    for comment in submission.submission_comments:
-      formatted_submission += f"""
-{comment["author_name"]} ({comment["created_at"]}):
-
-{comment["comment"]}
-"""
+      body = ""
+      for comment in submission.submission_comments:
+        body += f"{comment['author_name']} ({comment['created_at']}):\n\n"
+        body += comment["comment"] + "\n\n"
+      formatted_submission += format_section("Comments", body)
   except AttributeError:
     pass
-
   try:
     if submission.body:
-      formatted_submission += f"""
-
-# Body
-
-{submission.body}
-"""
+      formatted_submission += format_section("Body", submission.body)
+  except AttributeError:
+    pass
+  try:
+    if submission.submission_data:
+      formatted_submission += format_section(
+        "Quiz answers",
+        json.dumps(submission.submission_data, indent=2))
   except AttributeError:
     pass
-
   try:
     for attachment in submission.attachments:
       if "text/" not in attachment["content-type"]:
         continue
 
       contents = urllib.request.urlopen(attachment["url"]).read().decode("utf-8")
-      formatted_submission += f"""
+      formatted_submission += format_section(
+        attachment["filename"],
+        f"```\n{contents}\n```")
+  except AttributeError:
+    pass
+  try:
+    if submission.submission_history:
+      for prev_submission in submission.submission_history:
+        prev_submission = canvasapi.submission.Submission(
+          submission._requester, prev_submission)
+        prev_submission.assignment = submission.assignment
 
-## {attachment["filename"]}
-
-```
-{contents}
-```
-"""
+        formatted_submission += "\n\n" + format_submission(prev_submission)
   except AttributeError:
     pass
 
   return formatted_submission
+def format_section(title, body):
+  return f"\n# {title}\n\n{body}\n\n"
 def format_rubric(submission):
   """Format the rubric assessment of the `submission` in readable form."""
 
